/***************************************************************
* Name:         freqList.c
*
* Purpose:      Frequency list processing functions
*
* Course code:  21022105
* Student name: Marius Joldos
* E-mail:       Marius.Joldos@cs.utcluj.ro
****************************************************************
* Created:      2009-12-04
*
****************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "freqList.h"

/***********************************************************
* FUNCTION NAME: newNode
*
* DESCRIPTION:  creates a new node to hold the letter given as an argument
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* const char    letter    I  letter to store in the node
* RETURN VALUE: NodeT*       pointer to newly created node or
*                            NULL if unsuccessful
* CHANGES: -
*************************************************************/
NodeT* newNode(const char letter)
{
    NodeT *p = (NodeT *)malloc(sizeof(NodeT));
    if (p)
    {
        p->letter = letter;
        p->count = 1;
        p->next = 0;
    }
    return p;
}

/***********************************************************
* FUNCTION NAME: newList
*
* DESCRIPTION:  creates a new node to hold the letter given as an argument
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* RETURN VALUE: ListT*       pointer to newly created list header or
*                            NULL if unsuccessful
* CHANGES: -
*************************************************************/
ListT *newList()
{
    ListT* p = (ListT *)malloc(sizeof(ListT));
    if (p)
    {
        p->size = 0;
        p->first = 0;
    }
    return p;
}
/***********************************************************
* FUNCTION NAME: insertOrCount
*
* DESCRIPTION:  inserts the node if
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* ListT*        h
* NodeT*        pNode
* RETURN VALUE: int
*
* CHANGES: releases memory area pointed to by pNode if letter
*           already in list
*************************************************************/
int insertOrCount(ListT *h, NodeT *pNode)
{
    if (!h || !pNode) return 0;
    if (!h->first)
    { // empty list
        h->size++;
        h->first = pNode;
        return 1;
    }
    NodeT *pCurrent = h->first, *pPrevious = 0;
    // scan the list looking for a node which stores the
    // same letter as the one stored in node pointed to by pNode
    while (pCurrent)
    {
        if (pNode->letter < pCurrent->letter)
        { // insert node after node pointed to by pPrevious
            if (!pPrevious)
            { // insert as first node on list h
                pNode->next = h->first;
                h->first = pNode;
            }
            else
            {
                pNode->next = pPrevious->next;
                pPrevious->next = pNode;
            }
            h->size++;
            return 1;
        }
        else if (pNode->letter > pCurrent->letter)
        { // inspect next node
            pPrevious = pCurrent;
            pCurrent = pCurrent->next;
        }
        else
        { // letter already there
            pCurrent->count++;
            free(pNode);
            return 1;
        }
    }
    // node not on list; must be appended
    pPrevious->next = pNode;
    h->size++;
    return 1;
}
/***********************************************************
* FUNCTION NAME: printList
*
* DESCRIPTION:  prints a char count list
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* ListT*        h        I list to print
* RETURN VALUE: void
*
* CHANGES: -
*************************************************************/
void printList(const ListT *h)
{
    if (!h) return;
    printf("The list contains:\n");
    if (h->first)
    {
        for(NodeT *p = h->first; p; p = p->next)
            printf("%c %d\n", p->letter, p->count);
    }
    else printf("...nothing!\n");
}

/***********************************************************
* FUNCTION NAME: clearList
*
* DESCRIPTION: removes all nodes from list h
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* const ListT*  h          I list to clear
*               ;
* RETURN VALUE: void
*
* CHANGES: list h
*************************************************************/
void clearList(ListT *h)
{
    if (!h) return;
    while(h->first)
    {
        NodeT *p = h->first;
        h->first = h->first->next ;
        free(p);
    }
    h->size = 0;
}
